<?php

/**
 * @file
 * Rsstrack module
 */


/**
 * Implementation of  hook_form_FORM_ID_alter().
 */
function _rsstrack_form_system_rss_feeds_settings_alter(&$form, $form_state) {
  module_load_include('inc', 'rsstrack');

  $query = rsstrack_get_query();

  if (isset($form['#validate']) && is_array($form['#validate'])) {
    $form['#validate'][] =  'rsstrack_form_settings_validate';
  }
  else {
    $form['#validate'] =  array('rsstrack_form_settings_validate');
  }

  $form['buttons']['#weight'] = 20;

  $form['rsstrack'] = array(
    '#type' => 'fieldset',
    '#title' => t('RSS Tracking'),
    '#collapsible' => TRUE,
    '#collapsed' => empty($query) ? TRUE : FALSE,
  );
  $form['rsstrack']['preview'] = array(
    '#type' => 'item',
    '#title' => t('RSS node URL preview'),
    '#value' => '<code>'. url(t('some-node-title'), array('absolute' => TRUE, 'query' => $query)) .'</code>',
  );
  $form['rsstrack']['ga'] = array(
    '#type' => 'fieldset',
    '#title' => t('Google Analytics RSS Tracking'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#description' => t('If you want to use GA RSS tracking remember that <b>Campaign Source, Campaign Medium and Campaign Name are required by GA</b>.  For more details about these parameters, please go to <a href="@url">Google Analytics URL Builder</a> page.', array('@url' =>'http://www.google.com/support/googleanalytics/bin/answer.py?hl=en&answer=55578')),
  );
  $form['rsstrack']['ga']['rsstrack_utm_source'] = array(
    '#type' => 'textfield',
    '#title' => t('Campaign Source'),
    '#description' => t('<b>Required for GA</b>. The Referrer. Use <b>utm_source</b> to identify a search engine, newsletter name, or other source. <em>Example: google</em>'),
    '#default_value' => variable_get('rsstrack_utm_source', ''),
  );
  $form['rsstrack']['ga']['rsstrack_utm_medium'] = array(
    '#type' => 'textfield',
    '#title' => t('Campaign Medium'),
    '#description' => t('<b>Required for GA</b>. The marketing medium. Use <b>utm_medium</b> to identify a medium such as email or cost-per-click. <em>Example: cpc</em>'),
    '#default_value' => variable_get('rsstrack_utm_medium', ''),
  );
  $form['rsstrack']['ga']['rsstrack_utm_term'] = array(
    '#type' => 'textfield',
    '#title' => t('Campaign Term'),
    '#description' => t('Used for paid search. Use <b>utm_term</b> to note the keywords for this ad. <em>Example: running+shoes</em>'),
    '#default_value' => variable_get('rsstrack_utm_term', ''),
  );
  $form['rsstrack']['ga']['rsstrack_utm_content'] = array(
    '#type' => 'textfield',
    '#title' => t('Campaign Content'),
    '#description' => t('Use <b>utm_content</b> to differentiate ads or links that point to the same URL - for A/B testing and content-targeted ads. Examples: <em>logolink</em> or <em>textlink</em>'),
    '#default_value' => variable_get('rsstrack_utm_content', ''),
  );
  $form['rsstrack']['ga']['rsstrack_utm_campaign'] = array(
    '#type' => 'textfield',
    '#title' => t('Campaign Name'),
    '#description' => t('<b>Required for GA</b>. Product, promo code, or slogan. Used for keyword analysis. Use <b>utm_campaign</b> to identify a specific product promotion or strategic campaign. Example: <em>spring_sale</em>'),
    '#default_value' => variable_get('rsstrack_utm_campaign', ''),
  );
  $form['rsstrack']['rsstrack_custom'] = array(
    '#type' => 'textarea',
    '#title' => t('Custom tracking code'),
    '#description' => t('Write <b>not urlencoded</b> keys and values, one per line, without ampersands (&amp;). Values will be urlencoded and ampersands will be added automatically.<br>Example:<pre>my_value1=1<br>my_value2=my value<br>my_value3=[token]</pre>'),
    '#default_value' => variable_get('rsstrack_custom', ''),
  );
  if (module_exists('token')) {
    $form['rsstrack']['tokens'] = array(
      '#type' => 'item',
      '#title' => t('Tokens'),
      '#value' => theme('token_tree', array('node'), TRUE),
    );
  }
}

/**
 * A form_system_rss_feeds_settings validation handler
 *
 * @see form_system_rss_feeds_settings
 * @see rsstrack_form_system_rss_feeds_settings_alter
 */
function rsstrack_form_settings_validate($form, &$form_state) {
  // process GA
  $required_ga = FALSE;
  foreach (array('source', 'medium', 'term', 'content', 'campaign') as $param) {
    if (!empty($form_state['values']["rsstrack_utm_$param"])) {
      $required_ga = TRUE;
      break;
    }
  }
  if ($required_ga) {
    foreach (array('source', 'medium', 'campaign') as $param) {
      if (empty($form_state['values']["rsstrack_utm_$param"])) {
        form_set_error("rsstrack_utm_$param", t('When some GA RSS tracking field is non-empty you need to fill all required GA fields (Campaign Source, Campaign Medium, Campaign Campaign).'));
      }
    }
  }
}


/**
 * A generic function for generating RSS feeds from a set of nodes.
 *
 * @param $nids
 *   An array of node IDs (nid). Defaults to FALSE so empty feeds can be
 *   generated with passing an empty array, if no items are to be added
 *   to the feed.
 * @param $channel
 *   An associative array containing title, link, description and other keys.
 *   The link should be an absolute URL.
 *
 * Overrides node_feed()
 * @see node_feed()
 */
function rsstrack_node_feed($nids = FALSE, $channel = array()) {
  global $base_url, $language;

  if ($nids === FALSE) {
    $nids = array();
    $result = db_query_range(db_rewrite_sql('SELECT n.nid, n.created FROM {node} n WHERE n.promote = 1 AND n.status = 1 ORDER BY n.created DESC'), 0, variable_get('feed_default_items', 10));
    while ($row = db_fetch_object($result)) {
      $nids[] = $row->nid;
    }
  }

  $item_length = variable_get('feed_item_length', 'teaser');
  $namespaces = array('xmlns:dc' => 'http://purl.org/dc/elements/1.1/');

  $items = '';
  foreach ($nids as $nid) {
    // Load the specified node:
    $item = node_load($nid);
    $item->build_mode = NODE_BUILD_RSS;
    $item->link = url("node/$nid", array('absolute' => TRUE, 'query' => rsstrack_get_query($item)));

    if ($item_length != 'title') {
      $teaser = ($item_length == 'teaser') ? TRUE : FALSE;

      // Filter and prepare node teaser
      if (node_hook($item, 'view')) {
        $item = node_invoke($item, 'view', $teaser, FALSE);
      }
      else {
        $item = node_prepare($item, $teaser);
      }

      // Allow modules to change $node->content before the node is rendered.
      node_invoke_nodeapi($item, 'view', $teaser, FALSE);

      // Set the proper node property, then unset unused $node property so that a
      // bad theme can not open a security hole.
      $content = drupal_render($item->content);
      if ($teaser) {
        $item->teaser = $content;
        unset($item->body);
      }
      else {
        $item->body = $content;
        unset($item->teaser);
      }

      // Allow modules to modify the fully-built node.
      node_invoke_nodeapi($item, 'alter', $teaser, FALSE);
    }

    // Allow modules to add additional item fields and/or modify $item
    $extra = node_invoke_nodeapi($item, 'rss item');
    $extra = array_merge($extra, array(array('key' => 'pubDate', 'value' => gmdate('r', $item->created)), array('key' => 'dc:creator', 'value' => $item->name), array('key' => 'guid', 'value' => $item->nid .' at '. $base_url, 'attributes' => array('isPermaLink' => 'false'))));
    foreach ($extra as $element) {
      if (isset($element['namespace'])) {
        $namespaces = array_merge($namespaces, $element['namespace']);
      }
    }

    // Prepare the item description
    switch ($item_length) {
      case 'fulltext':
        $item_text = $item->body;
        break;
      case 'teaser':
        $item_text = $item->teaser;
        if (!empty($item->readmore)) {
          $item_text .= '<p>'. l(t('read more'), 'node/'. $item->nid, array('absolute' => TRUE, 'query' => rsstrack_get_query($item), 'attributes' => array('target' => '_blank'))) .'</p>';
        }
        break;
      case 'title':
        $item_text = '';
        break;
    }

    $items .= format_rss_item($item->title, $item->link, $item_text, $extra);
  }

  $channel_defaults = array(
    'version'     => '2.0',
    'title'       => variable_get('site_name', 'Drupal'),
    'link'        => $base_url,
    'description' => variable_get('site_mission', ''),
    'language'    => $language->language
  );
  $channel = array_merge($channel_defaults, $channel);

  $output = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
  $output .= "<rss version=\"". $channel["version"] ."\" xml:base=\"". $base_url ."\" ". drupal_attributes($namespaces) .">\n";
  $output .= format_rss_channel($channel['title'], $channel['link'], $channel['description'], $items, $channel['language']);
  $output .= "</rss>\n";

  drupal_set_header('Content-Type: application/rss+xml; charset=utf-8');
  print $output;
}

/**
 * Menu callback; displays all nodes associated with a term.
 *
 * Overrides: taxonomy_term_page()
 *
 * @see taxonomy_term_page()
 */
function rsstrack_taxonomy_term_page($str_tids = '', $depth = 0, $op = 'page') {
  module_load_include('inc', 'taxonomy', 'taxonomy.pages');
  $terms = taxonomy_terms_parse_string($str_tids);
  if ($terms['operator'] != 'and' && $terms['operator'] != 'or') {
    drupal_not_found();
  }

  if ($terms['tids']) {
    $result = db_query(db_rewrite_sql('SELECT t.tid, t.name FROM {term_data} t WHERE t.tid IN ('. db_placeholders($terms['tids']) .')', 't', 'tid'), $terms['tids']);
    $tids = array(); // we rebuild the $tids-array so it only contains terms the user has access to.
    $names = array();
    while ($term = db_fetch_object($result)) {
      $tids[] = $term->tid;
      $names[] = $term->name;
    }

    if ($names) {
      $title = implode(', ', $names);
      drupal_set_title(check_plain($title));

      switch ($op) {
        case 'page':
          // Build breadcrumb based on first hierarchy of first term:
          $current = new stdClass();
          $current->tid = $tids[0];
          $breadcrumb = array();
          while ($parents = taxonomy_get_parents($current->tid)) {
            $current = array_shift($parents);
            $breadcrumb[] = l($current->name, 'taxonomy/term/'. $current->tid);
          }
          $breadcrumb[] = l(t('Home'), NULL);
          $breadcrumb = array_reverse($breadcrumb);
          drupal_set_breadcrumb($breadcrumb);

          $output = theme('taxonomy_term_page', $tids, taxonomy_select_nodes($tids, $terms['operator'], $depth, TRUE));
          drupal_add_feed(url('taxonomy/term/'. $str_tids .'/'. $depth .'/feed'), 'RSS - '. $title);
          return $output;
          break;

        case 'feed':
          $channel['link'] = url('taxonomy/term/'. $str_tids .'/'. $depth, array('absolute' => TRUE));
          $channel['title'] = variable_get('site_name', 'Drupal') .' - '. $title;
          // Only display the description if we have a single term, to avoid clutter and confusion.
          if (count($tids) == 1) {
            $term = taxonomy_get_term($tids[0]);
            // HTML will be removed from feed description, so no need to filter here.
            $channel['description'] = $term->description;
          }

          $result = taxonomy_select_nodes($tids, $terms['operator'], $depth, FALSE);
          $items = array();
          while ($row = db_fetch_object($result)) {
            $items[] = $row->nid;
          }

          rsstrack_node_feed($items, $channel);
          break;

        default:
          drupal_not_found();
      }
    }
    else {
      drupal_not_found();
    }
  }
}

/**
 * Menu callback; displays an RSS feed containing recent blog entries of a given user.
 *
 * Overrides blog_feed_user()
 *
 * @see blog_feed_user()
 */
function rsstrack_blog_feed_user($account) {
  $result = db_query_range(db_rewrite_sql("SELECT n.nid, n.created FROM {node} n  WHERE n.type = 'blog' AND n.uid = %d AND n.status = 1 ORDER BY n.created DESC"), $account->uid, 0, variable_get('feed_default_items', 10));
  $channel['title'] = t("!name's blog", array('!name' => $account->name));
  $channel['link'] = url('blog/'. $account->uid, array('absolute' => TRUE));

  $items = array();
  while ($row = db_fetch_object($result)) {
    $items[] = $row->nid;
  }
  rsstrack_node_feed($items, $channel);
}

/**
 * Menu callback; displays an RSS feed containing recent blog entries of all users.
 *
 * Overrides blog_feed_last()
 *
 * @see blog_feed_last()
 */
function rsstrack_blog_feed_last() {
  $result = db_query_range(db_rewrite_sql("SELECT n.nid, n.created FROM {node} n WHERE n.type = 'blog' AND n.status = 1 ORDER BY n.created DESC"), 0, variable_get('feed_default_items', 10));
  $channel['title'] = t('!site_name blogs', array('!site_name' => variable_get('site_name', 'Drupal')));
  $channel['link'] = url('blog', array('absolute' => TRUE));

  $items = array();
  while ($row = db_fetch_object($result)) {
    $items[] = $row->nid;
  }

  rsstrack_node_feed($items, $channel);
}
